%%%%%% Emprirical Analysis %%%%%%%
% This part corresponds to Appendix A in Fan, Liao and Yao (2015)
%---------------------------------
% 1) Load Data
clear;
load('Data\SP500.mat');  %S&P 500 returns for diff. stocks
load('Data\FFmonth.mat');%Carhart 4 factors
load('Data\SP500PERMNO.mat'); % Company codes for different stocks

% 2) set up parameters
T = 60;
[row, col] = size(SP500);
nTest = col-T+1;  % number of tests
window = 1:nTest;
nWindow = length(window);

% recording
panelSize = zeros(nWindow,1); %describe how the panel size evolves with time
cardS = zeros(nWindow,1);   % number of stocks in screening set
cardSpctg = zeros(nWindow,1); % percentage of stocks in screening set

sComp = zeros(100,nWindow);  % to record selected components
Alpha_est = zeros(1000,nWindow); % record all alphas
sAlpha = zeros(100,nWindow); % alphas in the screening set


% 3) Testing

iter = 0;
for roll = window
    
    iter = iter+1;
    fprintf('Rolling window %s\n',num2str(iter));
    period = roll:(roll+T-1);
    %%%%%  Pull Data and Estimation   %%%%
    % find out stocks without missing data in given window
    RR = SP500(:,period);
    TF = isnan(RR);
    index = (sum(TF,2)==0);
    panelSize(iter) = sum(index);  % panel size
    N = panelSize(iter);
    
    % extracting panel data
    Rmkt_Rf = FFmonth(period,2);
    SMB = FFmonth(period,3);
    HML = FFmonth(period,4);
    Rf = FFmonth(period,5);
    Mom = FFmonth(period,6);
    YY = 100*RR(index,:)-ones(N,1)*Rf';    % excess returns (%) for 60 months
    
    % estimating alpha and betas
    FF = [Rmkt_Rf SMB HML Mom]';               % 4 by T
    X = [ones(T,1) Rmkt_Rf SMB HML Mom];
    ABBB = zeros(5,N);
    Alpha = zeros(N,1);
    Beta = zeros(N,4);
     
    for i = 1:N
        ABBB(:,i) = (X'*X)^(-1)*X'*YY(i,:)';
        Alpha(i) = ABBB(1,i);       % estimated alpha
        Beta(i,:) = ABBB(2:5,i)';    % estimated betas
    end
    
    %%%%%    Analysis     %%%%%
    % 1. covariance matrix
    Err = YY - repmat(Alpha,1,T) - Beta*FF;  % estimated u_t
    h_sigma_u = 1/(T-4)*(Err*Err');  % Sigma_u_hat, i.e. sample covariance matrix
    h_D = diag(diag(h_sigma_u));   % diagonal elements of sample covariance matrix w/ small sample correction
    
    % 2. Wald and PE component 
    
    delta_T = 0.9*sqrt(log(N)/T)*log(log(T));  % threshold on alpha  0.145
    std_alpha = abs(Alpha)./diag(h_D).^(1/2);
    S_hat = find(std_alpha > delta_T);    % selected alpha
    card_S = length(S_hat);         % cardinality
    Alpha_S = Alpha(S_hat);
    Sigma_S1=h_D(S_hat,S_hat);
    
    cardS(iter) = card_S;
    cardSpctg(iter) = card_S/N;
    
    
    permno = SP500PERMNO(index);
    sComp(1:card_S,roll) = permno(S_hat);
    sAlpha(1:card_S,roll) = Alpha_S;
    Alpha_est(1:N,roll) = Alpha;
    
    
end


% 4) Descriptive statistics summary

% cross sectional average of absolute alphas
alpha_CSavg = arrayfun(@(x) mean(abs(Alpha_est(1:panelSize(x),x))),window);
% cross sectional average of absolute alphas in the screening set
alpha_CSavg_screened = arrayfun(@(x) mean(abs(sAlpha(1:sum(sComp(:,x)>0),x))),window);
alpha_CSavg_screened = alpha_CSavg_screened(~isnan(alpha_CSavg_screened));  % some screen set might be empty

displaySummary = @(x) fprintf('%10.4f%10.4f%10.4f%10.4f%10.4f\n',...
    mean(x),std(x),median(x),min(x),max(x));
disp(repmat('-',1,60))
fprintf('---------- Descriptive statistics summary -----------\n')
fprintf('    Mean     Std.Dev.    Median      Min       Max\n')
displaySummary(panelSize)
displaySummary(cardS)
displaySummary(alpha_CSavg);
displaySummary(alpha_CSavg_screened);
disp(repmat('-',1,60))
fprintf('Variables: panelSize, cardS, alpha_CSavg, alpha_CSavg_screened\n')
disp(repmat('-',1,60))
