% Simulation -- error covariance is non-diagonal
% ------------------------------------------
% 1) Parameter setup
NN = 5000;               % repetitions
N = 500; T = 300;        % panel size
SigLevel = 0.95;        % significance level

%  H0 vs H1: to switch from H0 to H1, change alpha0
%  or, design TrueAlpha as you like
TrueAlpha = zeros(N,1);  % H_0
r = max(round(N/T),1);  %r = round(N^0.4);
alpha0 = 0;   %alpha0=sqrt(log(N)/(T));
TrueAlpha(1:r) = alpha0*ones(r,1);
% ----

% threshold for power enhancement
delta_T = 1.06*sqrt(log(N)/T)*log(log(T));  
% threshold for sparse error covariance
thr = 1.5;

% parameter for test in Fan (1996)
Fan_a = 1/(log(N)^2);
Fan_delta = sqrt(2*log(N*Fan_a));
Fan_mu = sqrt(2/pi)*Fan_a^(-1)*Fan_delta*(1+Fan_delta^(-2));
Fan_sigma = sqrt( sqrt(2/pi)* Fan_a^(-1)*Fan_delta^3*(1+3*Fan_delta^(-2)) );

% recording
rec_Wald = zeros(NN,1);
rec_WaldPE = zeros(NN,1);
rec_SS = zeros(NN,1);    % record whether screening set is empty
rec_Fan = zeros(NN,1); 
rec_FanPE = zeros(NN,1);

% 2) Test
for Rep = 1:NN

    if rem(Rep,50)==0
        disp(Rep)
    end
%%%%%  Calibration   %%%%

    % generate factor pricing model
    [FF, BB, UU, Sig0]=Jiawei_geneFPM2(N,T);

    %generating y_t  
    YY = repmat(TrueAlpha,1,T) + BB'*FF + UU;

%%%%%  Estimation   %%%%
    X=[ones(T,1) FF'];
    ABBB=zeros(4,N);
    Alpha=zeros(N,1);
    Beta=zeros(N,3);
    for i=1:N
        ABBB(:,i)=(X'*X)\X'*YY(i,:)';
        Alpha(i)=ABBB(1,i);       % estimated alpha
        Beta(i,:)=ABBB(2:4,i)';    % estimated betas
    end
    Err = YY - repmat(Alpha,1,T) - Beta*FF;
    h_sigma_u = 1/T*(Err*Err');         % sample covariance matrix
    h_D = T/(T-4)*diag(diag(h_sigma_u));% estimated diagonal matrix
    
    [h_Sig,rho2] = sparseMatrixEstimator(Err,thr); % estimated sparse matrix
%%%%% Construct test statistics %%%%
% 1) Wald
    v = T-4;  % degree of freedom
    P_F = eye(T)-FF'/(FF*FF')*FF;
    cc = ones(1,T)*P_F*ones(T,1);
    mu_wald = N*(T-4)/(T-6);
    sigma_wald = sqrt(2*N*(T-5)/(T-8))*(T-4)/(T-6);       
    sigma_wald_adj = sqrt(2*N*(v-1)/(v-4)*(1+(N-1)*rho2))*v/(v-2);
    % comment: the mean and variance are chosen to account for error
    % distribution, see Pesaran and Yamagata (2012)
    
    %WaldDiag=cc*Alpha'/h_D*Alpha;
    WaldTh = cc*Alpha'/h_Sig*Alpha;
    Wald = (WaldTh - mu_wald)/sigma_wald_adj;
    % Wald = (WaldTh - mu_wald)/sigma_wald;    this version is also applicable
% 2) PE component
    std_alpha = abs(Alpha)./diag(h_D).^(1/2);
    S_hat = find(std_alpha>delta_T);
    card_S = length(S_hat);
    Alpha_S = Alpha(S_hat);
    Sigma_S = h_D(S_hat,S_hat);
    J = cc*Alpha_S'/Sigma_S*Alpha_S;
    
% 3) Fan(1996)
    FanX = Alpha./sqrt(diag(h_D)/cc);
    FanH = sum(FanX.^2.*(abs(FanX) > Fan_delta));
    Fan_THtest = (FanH - Fan_mu)/Fan_sigma;
    
%%%%%   Recording   %%%%    
    rec_Wald(Rep) = (Wald > norminv(SigLevel,0,1));
    rec_WaldPE(Rep) = (Wald+J > norminv(SigLevel,0,1));
    rec_SS(Rep) = (J==0);
    rec_Fan(Rep) = (Fan_THtest > norminv(SigLevel,0,1));
    rec_FanPE(Rep) = (Fan_THtest+J > norminv(SigLevel,0,1));
end

p_Wald = sum(rec_Wald)/NN;
p_WaldPE = sum(rec_WaldPE)/NN;
p_S = sum(rec_SS)/NN;
p_Fan = sum(rec_Fan)/NN;
p_FanPE = sum(rec_FanPE)/NN;

% printing result
disp('------------  Non-diagonal error covariance  ----------------')
fprintf('N: %s, T: %s, alpha0: %s, replication: %s\n',num2str(N),num2str(T),num2str(alpha0),num2str(NN));
fprintf('p-values: Wald     WaldPE     Fan     FanPE     S\n');
fprintf('    %10.4f%10.4f%10.4f%10.4f%10.4f\n',p_Wald,p_WaldPE,p_Fan,p_FanPE,p_S);







