
% Testing cross-sectional independence in Fan, Liao and Yao (2015)
% Section 6.2

%-------------------------------------
% 1) Set up parameters
n = 200; T = 100;   % panel size
Hypothesis = 1;     % determine null and alternative

rep = 1000;           % replications 2000
N = n*(n-1)/2;        % number of parameters
nBlock = floor(n^.3);  % number of blocks in the covariance
BlockSize = 4; 
rho = 0.2; 
cc = 1.5;           % threshold constant
Hetero = 1;         % Heteroscadasiticity in covariance

rate2 = 1.5*log(N)*(log(log(T)))^2;
tt = rate2/T;

% recording
QuadTest = zeros(rep,1);
QuadPETest = zeros(rep,1);
SS = zeros(rep,1);
AvgMaxSignal = zeros(rep,1);

%---------------------------------------- 
% 2) Set up hypothesis
if(Hypothesis)
    RR = BlockGen(nBlock,BlockSize,rho,n);  % Constructing Block Diagonal
else
    RR = eye(n);
end

%-----------------------------------------
% 3) Testing
for irep = 1:rep
    if rem(irep,50)==0
        disp(irep)
    end
   %%%%  data generation  %%%%
     
    mu1 = randn(n,1)*sqrt(0.25);
    mu = mu1*ones(1,T);
    % set up X as AR processes
    X = zeros(n,T);
    X(:,1)=5*ones(n,1);
    for ix = 2:T
        X(:,ix) = X(:,ix-1)*0.7 + mu1 + randn(n,1);
    end
    % heteroskedastic errors
    if(Hetero)
     IndvVar = (0.5*mean(X,2)+1).^2;
     ScaleVar = sqrt(IndvVar/mean(IndvVar));
     e = RR'*diag(ScaleVar)*randn(n,T);
    else
     e = RR'*randn(n,T);
    end
    % generate response
    Y = -ones(n,T)+2*X+mu+e;
    
    %%%%  estimation  %%%%
    tildeY = Y-mean(Y,2)*ones(1,T);
    tildeX = X-mean(X,2)*ones(1,T);
    B = reshape(tildeX,n*T,1)'*reshape(tildeY,n*T,1);
    A = reshape(tildeX,n*T,1)'*reshape(tildeX,n*T,1);
    beta = B/A;
    tildeU = tildeY - tildeX*beta;  % n by T
    Su = tildeU*tildeU'/(T-1);
    R = corrcov(Su);
    
    
    %%%% constructing test  %%%%
    
    % 1. quadratic test
    sum1 = (norm(R,'fro')^2-n)/2;
    J1 = (sum1*T-N)/sqrt(n*(n-1))-n/(2*(T-1));
    QuadTest(irep) = J1>norminv(0.95);
    
    % 2. power enhancement
    off = R-eye(n);
    num = off.^2;
    denom = (ones(n,n)-num).^2;
    A1 = num./denom;  % rho^2/ (1-rho^2)^2 
    AvgMaxSignal(irep) = max(max(A1));
    A2 = A1.*(A1>cc*rate2/T);  % rho^2/ (1-rho^2)^2 *  1{ rho^2/ (1-rho^2)^2>delta^2/T}
    J0 = sum(sum(A2))/2*T*sqrt(N);
    QuadPETest(irep) = J0+J1>norminv(0.95);
    
    % 3. screening set
    SS(irep) = (J0==0);
     
end;

 
fprintf('n: %s  ,T: %s,  H: %s\n',num2str(n),num2str(T),num2str(Hypothesis));
fprintf('p-values:  Quad       Quad+PE      SS\n')
fprintf('       %10.4f%10.4f%10.4f\n',sum(QuadTest)/rep,sum(QuadPETest)/rep,sum(SS)/rep);




